@@ -4,18 +4,14 @@ from typing import Optional, Any, List, Dict, TypeVar, Generic
 from datetime import datetime, UTC
 from uuid import uuid4
 from pymongo.errors import PyMongoError, DuplicateKeyError, OperationFailure
-from abs_exception_core.exceptions import NotFoundError, GenericHttpError, BadRequestError
-import logging
-import re
-
-logger = logging.getLogger(__name__)
+from abs_exception_core.exceptions import GenericHttpError, BadRequestError, DuplicatedError,PermissionDeniedError
+from pymongo import ASCENDING
 
 T = TypeVar('T')
 
 class BaseCollectionRepository(Generic[T]):
     """
     Base repository class for MongoDB collections with common CRUD operations.
-    Provides robust error handling, input validation, and transaction support.
     """
 
     def __init__(self, db: AsyncIOMotorDatabase):
@@ -63,7 +59,6 @@ class BaseCollectionRepository(Generic[T]):
             BadRequestError: For invalid operations
             NotFoundError: For missing resources
         """
-        logger.error(f"Error during {operation}: {str(error)}")
         
         if isinstance(error, DuplicateKeyError):
             raise BadRequestError(
@@ -103,22 +98,23 @@ class BaseCollectionRepository(Generic[T]):
             collection = self.db[collection_name]
             result = await collection.update_many(
                 {column.column_field: {"$exists": False}},
-                {"$set": {column.column_field: column.column_default or self.get_default_for_type(column.column_type)}},
-                hint="_id_"
+                {"$set": {column.column_field: column.column_default or self.get_default_for_type(column.column_type)}}
             )
-            logger.info(f"Added field {column.column_field} to {result.modified_count} documents")
+            if column.column_index:
+                await collection.create_index(column.column_field)
             return True
                     
         except Exception as e:
             await self._handle_mongo_error("add field", e)
 
-    async def delete_field(self, column_name: str, collection_name: Optional[str] = None) -> bool:
+    async def delete_field(self, column_name: str, collection_name: Optional[str] ,protected_fields:Optional[List[str]]=None) -> bool:
         """
         Delete a field from all documents in the collection.
         
         Args:
             column_name (str): Name of the field to delete
             collection_name (Optional[str]): Name of the collection
+            protected_fields (List[str]): List of protected fields
             
         Returns:
             bool: True if operation was successful
@@ -128,6 +124,13 @@ class BaseCollectionRepository(Generic[T]):
         """
         if not collection_name:
             raise BadRequestError(detail="Collection name is required")
+        if protected_fields is None:
+            protected_fields = ["_id","uuid","created_at","updated_at","name"]
+        else:
+            protected_fields = protected_fields+["_id","uuid","created_at","updated_at","name"]
+
+        if protected_fields and column_name in protected_fields:
+            raise PermissionDeniedError("Requested field cannot be deleted.")
             
         try:
             collection = self.db[collection_name]
@@ -136,12 +139,51 @@ class BaseCollectionRepository(Generic[T]):
                 {"$unset": {column_name: ""}}
             )
             
-            logger.info(f"Deleted field {column_name} from {result.modified_count} documents")
             return True
                     
         except Exception as e:
             await self._handle_mongo_error("delete field", e)
 
+    async def rename_field(self, old_name: str, new_name: str, collection_name: Optional[str] = None) -> bool:
+        """
+        Rename a field in all documents in the collection.
+        
+        Args:
+            old_name (str): Current name of the field
+            new_name (str): New name of the field
+            collection_name (Optional[str]): Name of the collection
+            
+        Returns:
+            bool: True if operation was successful
+            
+        Raises:
+            BadRequestError: If collection name is invalid or target field already exists
+        """
+        if not collection_name:
+            raise BadRequestError(detail="Collection name is required")
+        
+        try:
+            collection = self.db[collection_name]
+            
+            # Check if target field exists in any records
+            existing_docs = await collection.find({new_name: {"$exists": True}}).to_list(length=1)
+            if existing_docs:
+                raise DuplicatedError(
+                    detail=f"Cannot rename field to '{new_name}' as it already exists in some records"
+                )
+            
+            result = await collection.update_many(
+                {},
+                {"$rename": {old_name: new_name}}
+            )
+            
+            return True
+                    
+        except Exception as e:
+            await self._handle_mongo_error("rename field", e)
+            
+    
+
     async def create_collection(self, collection_data: CreateCollectionSchema) -> bool:
         """
         Create a new collection with default values.
@@ -159,10 +201,14 @@ class BaseCollectionRepository(Generic[T]):
         try:
             collection = self.db[collection_data.collection_name]
             default_fields = collection_data.default_values or {}
-            full_doc = {**self.get_base_document_fields(), **default_fields}
+            base_fields = self.get_base_document_fields()
+            full_doc = {**base_fields, **default_fields}
+
+            collection.create_index([("_id", ASCENDING)])
+            collection.create_index([("uuid", ASCENDING)])
+
             
             await collection.insert_one(full_doc)
-            logger.info(f"Created collection {collection_data.collection_name}")
             return True
                     
         except Exception as e:
@@ -181,7 +227,6 @@ class BaseCollectionRepository(Generic[T]):
         
         try:
             await self.db.drop_collection(collection_name)
-            logger.info(f"Deleted collection {collection_name}")
             return True
             
         except Exception as e:
@@ -214,7 +259,6 @@ class BaseCollectionRepository(Generic[T]):
                 "dropTarget": False 
             })
 
-            logger.info(f"Renamed collection from {old_name} to {new_name}")
             return True
             
         except Exception as e:
@@ -250,8 +294,7 @@ class BaseCollectionRepository(Generic[T]):
             FieldTypeEnum.FLOAT: 0.0,
             FieldTypeEnum.LIST: [],
             FieldTypeEnum.DICT: {},
-            FieldTypeEnum.DATETIME: datetime.now(UTC)
+            FieldTypeEnum.DATETIME: datetime.now(UTC),
+            FieldTypeEnum.DATE : datetime.now(UTC)
         }
         return default_values.get(field_type) or ""
-
-
