@@ -32,9 +32,11 @@ from ..schema import (
     AggregationType,
     ReferenceField,
     FilterSchema,
+    SortSchema,
+    ReferenceFieldType,
+    AggregationStage
 )
-from ..util.operator_mappings import logical_operator_map, apply_condition
-from ..util.coerce_value import coerce_value
+from ..util import coerce_value,apply_condition,logical_operator_map
 
 T = TypeVar("T", bound=BaseModel)
 DocumentType = TypeVar("DocumentType", bound=Document)
@@ -224,6 +226,15 @@ class BaseRepository:
         except Exception as e:
             await self._handle_mongo_error("bulk create documents", e)
 
+    async def bulk_write(self, data: List[T], collection_name: Optional[str] = None):
+        """Bulk write documents to the collection"""
+        try:  
+            collection = self.get_collection(collection_name)
+            result = await collection.bulk_write(data,ordered=False)
+            return result.upserted_count
+        except Exception as e:
+            await self._handle_mongo_error("bulk write documents", e)
+
     async def update(
         self, id: Union[str, ObjectId], obj: T, collection_name: Optional[str] = None
     ) -> Dict[str, Any]:
@@ -353,6 +364,15 @@ class BaseRepository:
         except Exception as e:
             await self._handle_mongo_error("get many documents", e)
 
+    
+    def _build_search_conditions(
+        self, search_term: str, searchable_fields: List[str]
+    ) -> List[Dict[str, Any]]:
+        """Build case-insensitive search conditions for fields"""
+        regex = {"$regex": f".*{search_term}.*", "$options": "i"}
+        return [{field: regex} for field in searchable_fields]
+    
+
     def _build_query_filter(
         self, find: ListFilter, collection_name: Optional[str] = None
     ) -> Optional[Dict[str, Any]]:
@@ -382,19 +402,6 @@ class BaseRepository:
 
         return base_filter
 
-    def _build_search_conditions(
-        self, search_term: str, searchable_fields: List[str]
-    ) -> List[Dict[str, Any]]:
-        """Build case-insensitive search conditions for fields"""
-        regex = {"$regex": f".*{search_term}.*", "$options": "i"}
-        return [{field: regex} for field in searchable_fields]
-
-    def _get_sort_order(self, sort_order: List[Dict[str, Any]]) -> List[tuple]:
-        """Generate sort tuples for MongoDB"""
-        return [
-            (s.field, ASCENDING if s.direction == SortDirection.ASC else DESCENDING)
-            for s in sort_order
-        ] or [("_id", DESCENDING)]
 
     def _build_filter_condition(
         self,
@@ -432,6 +439,14 @@ class BaseRepository:
 
         return {}
 
+
+    def _get_sort_order(self, sort_order: List[SortSchema]) -> List[tuple]:
+        """Generate sort tuples for MongoDB"""
+        return [
+            (s.field, ASCENDING if s.direction == SortDirection.ASC else DESCENDING)
+            for s in sort_order
+        ] or [("_id", DESCENDING)]
+
     async def get_all(
         self, find: ListFilter, collection_name: Optional[str] = None
     ) -> List[Dict[str, Any]]:
@@ -500,20 +515,6 @@ class BaseRepository:
             },
         }
 
-    def _get_field_rule(
-        self,
-        field_rules: List[FieldRule],
-        pipeline: List[Dict[str, Any]],
-        collection_name: Optional[str] = None,
-    ) -> List[Dict[str, Any]]:
-        """Append field modifier stages to the aggregation pipeline."""
-        if not field_rules:
-            return pipeline
-
-        field_rule_stages = self._build_field_rules_stage(field_rules, collection_name)
-        pipeline.extend(field_rule_stages)
-        return pipeline
-
     def _build_field_rules_stage(
         self, field_rules: List[FieldRule], collection_name: Optional[str] = None
     ) -> List[Dict[str, Any]]:
@@ -525,11 +526,11 @@ class BaseRepository:
             field_path = rule.field
             conditions = rule.conditions
 
-            if not conditions or not conditions.conditions:
-                continue
-
             if action not in ("include", "exclude"):
                 raise ValueError(f"Unsupported filter action: {action}")
+            
+            if not conditions or not conditions.conditions:
+                continue
 
             condition_expr = self._build_filter_condition(
                 filter_dict=conditions.model_dump(),
@@ -537,9 +538,7 @@ class BaseRepository:
                 is_expr=True,
             )
 
-            filter_condition = (
-                {"$not": [condition_expr]} if action == "exclude" else condition_expr
-            )
+            filter_condition = ( {"$not": [condition_expr]} if action == "exclude" else condition_expr)
 
             stages.append(
                 {
@@ -556,137 +555,141 @@ class BaseRepository:
             )
 
         return stages
+    
+    def _get_field_rule(
+        self,
+        field_rules: List[FieldRule],
+        pipeline: List[Dict[str, Any]],
+        collection_name: Optional[str] = None,
+    ) -> List[Dict[str, Any]]:
+        """Append field modifier stages to the aggregation pipeline."""
+        if not field_rules:
+            return pipeline
+
+        field_rule_stages = self._build_field_rules_stage(field_rules, collection_name)
+        pipeline.extend(field_rule_stages)
+        return pipeline
+
+    
 
     def _build_reference_stage(
-        self, pipeline: List[Dict[str, Any]], references: List[ReferenceField]
+        self, pipeline: List[Dict[str, Any]], references: ReferenceFieldType
     ) -> List[Dict[str, Any]]:
         """Build reference stages supporting both flat and deeply nested alias paths."""
+        if isinstance(references,AggregationStage):
+            return pipeline+references.pipeline
+        else:
+            def process_reference(ref, current_pipeline):
+                local_path = ref.local_field
+                foreign_field = ref.foreign_field or "_id"
+                from_collection = ref.foreign_entity
+                alias = ref.alias or local_path.split(".")[-1] + "_info"
+                view_fields = ref.view_fields or []
+                is_single = ref.single_object or False
+                sort_fields = ref.sort_fields or []
 
-        def process_reference(ref, current_pipeline):
-            local_path = ref.local_field
-            foreign_field = ref.foreign_field or "_id"
-            from_collection = ref.foreign_entity
-            alias = ref.alias or local_path.split(".")[-1] + "_info"
-            view_fields = ref.view_fields or []
-            is_single = ref.single_object or False
-            sort_fields = ref.sort_fields or []
+                # Convert sort_fields to MongoDB sort dict
+                ref_sort_dict = {}
+                for sort_item in sort_fields:
+                    direction = 1 if sort_item.direction == SortDirection.ASC else -1
+                    ref_sort_dict[sort_item.field] = direction
 
-            # Convert sort_fields to MongoDB sort dict
-            ref_sort_dict = {}
-            for sort_item in sort_fields:
-                direction = 1 if sort_item.direction == SortDirection.ASC else -1
-                ref_sort_dict[sort_item.field] = direction
+                path_parts = local_path.split(".")
+                is_nested_local = len(path_parts) > 1
+                alias_parts = alias.split(".")
+                is_nested_alias = len(alias_parts) > 1
 
-            path_parts = local_path.split(".")
-            is_nested_local = len(path_parts) > 1
-            alias_parts = alias.split(".")
-            is_nested_alias = len(alias_parts) > 1
+                local_key = path_parts[-1]
+                embed_field = alias_parts[-1]
 
-            local_key = path_parts[-1]
-            embed_field = alias_parts[-1]
-
-            # Prepare lookup pipeline
-            lookup_pipeline = [
-                {
-                    "$match": {
-                        "$expr": {
-                            "$in": [
-                                f"${foreign_field}",
-                                {
-                                    "$cond": {
-                                        "if": {"$eq": [{"$type": "$$ids"}, "array"]},
-                                        "then": "$$ids",
-                                        "else": ["$$ids"],
-                                    }
-                                },
-                            ]
+                # Prepare lookup pipeline
+                lookup_pipeline = [
+                    {
+                        "$match": {
+                            "$expr": {
+                                "$in": [
+                                    f"${foreign_field}",
+                                    {
+                                        "$cond": {
+                                            "if": {"$eq": [{"$type": "$$ids"}, "array"]},
+                                            "then": "$$ids",
+                                            "else": ["$$ids"],
+                                        }
+                                    },
+                                ]
+                            }
                         }
                     }
-                }
-            ]
-            if view_fields:
-                lookup_pipeline.append({"$project": {f: 1 for f in view_fields}})
-            if ref_sort_dict:
-                lookup_pipeline.append({"$sort": ref_sort_dict})
+                ]
+                
+                if ref_sort_dict:
+                    lookup_pipeline.append({"$sort": ref_sort_dict})
+                if view_fields:
+                    lookup_pipeline.append({"$project": {f: 1 for f in view_fields}})
 
-            joined_var = f"_joined_{embed_field}"
+                joined_var = f"_joined_{embed_field}"
 
-            # Step 1: Extract IDs
-            if is_nested_local:
-                id_var = f"_ids_{embed_field}"
+                # Step 1: Extract IDs
+                if is_nested_local:
+                    id_var = f"_ids_{embed_field}"
+                    current_pipeline.append(
+                        {
+                            "$set": {
+                                id_var: self._build_dynamic_id_extraction(
+                                    path_parts[:-1], local_key
+                                )
+                            }
+                        }
+                    )
+                else:
+                    id_var = local_path
+
+                # Step 2: Lookup
                 current_pipeline.append(
                     {
-                        "$set": {
-                            id_var: self._build_dynamic_id_extraction(
-                                path_parts[:-1], local_key
-                            )
+                        "$lookup": {
+                            "from": from_collection,
+                            "let": {"ids": f"${id_var}"},
+                            "pipeline": lookup_pipeline,
+                            "as": joined_var,
                         }
                     }
                 )
-            else:
-                id_var = local_path
 
-            # Step 2: Lookup
-            current_pipeline.append(
-                {
-                    "$lookup": {
-                        "from": from_collection,
-                        "let": {"ids": f"${id_var}"},
-                        "pipeline": lookup_pipeline,
-                        "as": joined_var,
-                    }
-                }
-            )
-
-            # Step 3: Embed result
-            if is_nested_alias:
-                embed_expr = self._build_dynamic_embedding(
-                    alias_parts, joined_var, foreign_field, local_key
-                )
-                if is_single:
-                    current_pipeline.append(
-                        {
-                            "$set": {
-                                alias_parts[0]: {
-                                    "$cond": {
-                                        "if": {"$gt": [{"$size": f"${joined_var}"}, 0]},
-                                        "then": {"$arrayElemAt": [f"${joined_var}", 0]},
-                                        "else": None,
-                                    }
-                                }
-                            }
-                        }
+                # Step 3: Embed result
+                if is_nested_alias:
+                    embed_expr = self._build_dynamic_embedding(
+                        alias_parts, joined_var, foreign_field, local_key
                     )
-                else:
                     current_pipeline.append({"$set": {alias_parts[0]: embed_expr}})
-            else:
-                if is_single:
-                    current_pipeline.append(
-                        {
-                            "$set": {
-                                alias: {
-                                    "$cond": {
-                                        "if": {"$gt": [{"$size": f"${joined_var}"}, 0]},
-                                        "then": {"$arrayElemAt": [f"${joined_var}", 0]},
-                                        "else": None,
+                else:
+                    if is_single:
+                        current_pipeline.append(
+                            {
+                                "$set": {
+                                    alias: {
+                                        "$cond": {
+                                            "if": {"$gt": [{"$size": f"${joined_var}"}, 0]},
+                                            "then": {"$arrayElemAt": [f"${joined_var}", 0]},
+                                            "else": None,
+                                        }
                                     }
                                 }
                             }
-                        }
-                    )
-                else:
-                    current_pipeline.append({"$set": {alias: f"${joined_var}"}})
+                        )
+                    else:
+                        current_pipeline.append({"$set": {alias: f"${joined_var}"}})
 
-            if is_nested_local:
-                current_pipeline.append({"$unset": [id_var]})
-            current_pipeline.append({"$unset": [joined_var]})
+                if is_nested_local:
+                    current_pipeline.append({"$unset": [id_var]})
+                current_pipeline.append({"$unset": [joined_var]})
 
-            return current_pipeline
+                return current_pipeline
 
-        for ref in references:
-            pipeline = process_reference(ref, pipeline)
+            for ref in references:
+                pipeline = process_reference(ref, pipeline)
 
-        return pipeline
+            return pipeline
 
     def _build_dynamic_id_extraction(self, path_parts, local_key):
         """Build dynamic ID extraction for any depth with safe $map/$reduce."""
@@ -772,22 +775,13 @@ class BaseRepository:
         joined_var: str,
         foreign_field: str,
         local_key: str,
-        is_single: bool = False,
     ) -> Any:
         """Wrapper to initiate nested embedding from top-level alias"""
         if len(alias_parts) == 1:
-            if is_single:
-                return {
-                    "$cond": {
-                        "if": {"$gt": [{"$size": f"${joined_var}"}, 0]},
-                        "then": {"$arrayElemAt": [f"${joined_var}", 0]},
-                        "else": None,
-                    }
-                }
             return f"${alias_parts[0]}"  # Just return if there's no nesting
 
         return self._build_nested_embedding(
-            f"${alias_parts[0]}", alias_parts[1:], joined_var, foreign_field, local_key, is_single
+            f"${alias_parts[0]}", alias_parts[1:], joined_var, foreign_field, local_key
         )
 
     def _build_nested_embedding(
@@ -797,7 +791,6 @@ class BaseRepository:
         joined_var: str,
         foreign_field: str,
         local_key: str,
-        is_single: bool = False,
     ) -> Dict[str, Any]:
         """Recursively build nested embedding expressions with safe map for any depth."""
 
@@ -826,28 +819,18 @@ class BaseRepository:
                                 field: {
                                     "$cond": {
                                         "if": {"$isArray": f"${joined_var}"},
-                                        "then": (
-                                            {
-                                                "$cond": {
-                                                    "if": {"$gt": [{"$size": f"${joined_var}"}, 0]},
-                                                    "then": {"$arrayElemAt": [f"${joined_var}", 0]},
-                                                    "else": None,
-                                                }
+                                        "then": {
+                                            "$filter": {
+                                                "input": f"${joined_var}",
+                                                "as": "j",
+                                                "cond": {
+                                                    "$eq": [
+                                                        f"$$j.{foreign_field}",
+                                                        f"$$parent.{local_key}",
+                                                    ]
+                                                },
                                             }
-                                            if is_single
-                                            else {
-                                                "$filter": {
-                                                    "input": f"${joined_var}",
-                                                    "as": "j",
-                                                    "cond": {
-                                                        "$eq": [
-                                                            f"$$j.{foreign_field}",
-                                                            f"$$parent.{local_key}",
-                                                        ]
-                                                    },
-                                                }
-                                            }
-                                        ),
+                                        },
                                         "else": [],
                                     }
                                 }
@@ -872,7 +855,6 @@ class BaseRepository:
                                 joined_var,
                                 foreign_field,
                                 local_key,
-                                is_single,
                             )
                         },
                     ]
