@@ -1,11 +1,8 @@
-
 from pydantic import BaseModel
-from typing import Optional, Any, List, Dict
-from datetime import datetime, UTC
-from uuid import uuid4
+from typing import Optional, Any, Dict
+from datetime import UTC
 from enum import Enum
 
-
 class FieldTypeEnum(str, Enum):
     STR = "str"
     INT = "int"
@@ -14,11 +11,14 @@ class FieldTypeEnum(str, Enum):
     LIST = "list"
     DICT = "dict"
     DATETIME = "datetime"
+    DATE = "date"
+
 # Add column field schema
 class AddColumnField(BaseModel):
     column_field:str
     column_type:FieldTypeEnum
     column_default:Optional[Any] = None
+    column_index:Optional[bool] = False
 
 class CreateCollectionSchema(BaseModel):
     collection_name:str
