@@ -14,11 +14,17 @@ class Operator(str, Enum):
     NIN = "nin"
     LIKE = "like"
     ILIKE = "ilike"
+    NOT_LIKE = "not_like"
+    NOT_ILIKE = "not_ilike"
     BETWEEN = "between"
     IS_NULL = "is_null"
     IS_NOT_NULL = "is_not_null"
     MIN_LENGTH = "min_length"
     MAX_LENGTH = "max_length"
+    STARTS_WITH = "starts_with"
+    NOT_STARTS_WITH = "not_starts_with"
+    ENDS_WITH = "ends_with"
+    NOT_ENDS_WITH = "not_ends_with"
 
 class LogicalOperator(str, Enum):
     AND = "and"
@@ -139,7 +145,13 @@ class ReferenceField(BaseModel):
     use_graph_lookup: Optional[GraphLookup] = Field(default=None)
     sort_fields: Optional[List[SortSchema]] = Field(default=None, description="Fields to sort the referenced documents by")
     single_object: Optional[bool] = Field(default=False, description="If true, the referenced document will be returned as a single object")
-    
+
+# Type for aggregation pipeline stages
+class AggregationStage(BaseModel):
+    pipeline: list = Field(description="The pipeline configuration for the stage")
+
+# Union type for reference fields that can be either ReferenceField or AggregationStage
+ReferenceFieldType = Union[List[ReferenceField], AggregationStage]
 
 class ListFilter(BaseModel):
     pre_filters: Optional[FilterSchema] = None
@@ -150,5 +162,5 @@ class ListFilter(BaseModel):
     search: Optional[str] = None
     searchable_fields: Optional[List[str]] = None
     field_rules: Optional[List[FieldRule]] = None
-    reference_fields: Optional[List[ReferenceField]] = None
+    reference_fields: Optional[ReferenceFieldType] = None
     aggregations: Optional[Aggregations] = None
\ No newline at end of file
