@@ -1,3 +1,5 @@
+from typing import List
+
 from ..repository.base_collection_repository import BaseCollectionRepository
 from ..schema import CreateCollectionSchema,AddColumnField
 
@@ -17,13 +19,12 @@ class BaseCollectionService:
     async def rename_collection(self, old_name: str, new_name: str):
         return await self.repository.rename_collection(old_name, new_name)
     
-    async def add_field(self, collection_name: str, field: AddColumnField):
-        return await self.repository.add_field(collection_name, field)
+    async def add_field(self, field: AddColumnField, collection_name: str):
+        return await self.repository.add_field( field,collection_name)
     
-    async def delete_field(self, collection_name: str, field: str):
-        return await self.repository.delete_field(field,collection_name)
-    
-    async def get_collection_names(self):
-        return await self.repository.get_collection_names()
+    async def delete_field(self, field: str, collection_name: str,protected_fields:List[str]):
+        return await self.repository.delete_field(field,collection_name,protected_fields)
     
+    async def rename_field(self, old_name: str, new_name: str, collection_name: str):
+        return await self.repository.rename_field(old_name, new_name, collection_name)
     
\ No newline at end of file
