@@ -69,6 +69,53 @@ def apply_condition(model, operator: Operator, field: str, value: Any, is_expr:
                 }
             }
 
+        elif operator == Operator.NOT_ILIKE:
+            return {
+                "$not": {
+                    "$regexMatch": {
+                        "input": field,
+                        "regex": f".*{value}.*",
+                        "options": "i"
+                    }
+                }
+            }
+
+        elif operator == Operator.STARTS_WITH:
+            return {
+                "$regexMatch": {
+                    "input": field,
+                    "regex": f"^{value}"
+                }
+            }
+
+        elif operator == Operator.NOT_STARTS_WITH:
+            return {
+                "$not": {
+                    "$regexMatch": {
+                        "input": field,
+                        "regex": f"^{value}"
+                    }
+                }
+            }
+
+        elif operator == Operator.ENDS_WITH:
+            return {
+                "$regexMatch": {
+                    "input": field,
+                    "regex": f"{value}$"
+                }
+            }
+
+        elif operator == Operator.NOT_ENDS_WITH:
+            return {
+                "$not": {
+                    "$regexMatch": {
+                        "input": field,
+                        "regex": f"{value}$"
+                    }
+                }
+            }
+
         elif operator == Operator.IS_NULL:
             return {"$eq": [field, None]}
         elif operator == Operator.IS_NOT_NULL:
@@ -132,6 +179,16 @@ def apply_condition(model, operator: Operator, field: str, value: Any, is_expr:
             return {field: {"$regex": f".*{value}.*"}}
         elif operator == Operator.ILIKE:
             return {field: {"$regex": f".*{value}.*", "$options": "i"}}
+        elif operator == Operator.NOT_ILIKE:
+            return {field: {"$not": {"$regex": f".*{value}.*", "$options": "i"}}}
+        elif operator == Operator.STARTS_WITH:
+            return {field: {"$regex": f"^{value}"}}
+        elif operator == Operator.NOT_STARTS_WITH:
+            return {field: {"$not": {"$regex": f"^{value}"}}}
+        elif operator == Operator.ENDS_WITH:
+            return {field: {"$regex": f"{value}$"}}
+        elif operator == Operator.NOT_ENDS_WITH:
+            return {field: {"$not": {"$regex": f"{value}$"}}}
         elif operator == Operator.BETWEEN:
             if isinstance(value, list) and len(value) == 2:
                 return {field: {"$gte": value[0], "$lte": value[1]}}
